@@ -537,9 +537,9 @@ hsh_rush(struct dstat *ds, struct objhead *oh)
 		DSL(DBG_WAITINGLIST, req->vsl->wid, "off waiting list");
 		if (SES_ScheduleReq(req)) {
 			/*
-			 * We could not schedule the session, leave the
-			 * rest on the busy list.
+			 * We could not schedule the session, put it back.
 			 */
+			VTAILQ_INSERT_HEAD(&wl->list, req, w_list);
 			break;
 		}
 	}
@@ -559,49 +559,61 @@ HSH_Purge(struct worker *wrk, struct objhead *oh, double ttl, double grace,
 double keep)
 {
 	struct objcore *oc, **ocp;
-	unsigned spc, nobj, n;
+	unsigned spc, ospc, nobj, n;
 	struct object *o;
+	int more = 0;
+
 	double now;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
-	spc = WS_Reserve(wrk->aws, 0);
-	ocp = (void*)wrk->aws->f;
-	Lck_Lock(&oh->mtx);
-	assert(oh->refcnt > 0);
-	nobj = 0;
-	now = VTIM_real();
-	VTAILQ_FOREACH(oc, &oh->objcs, list) {
-		CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-		assert(oc->objhead == oh);
-		if (oc->flags & OC_F_BUSY) {
-			/*
-			 * We cannot purge busy objects here, because their
-			 * owners have special rights to them, and may nuke
-			 * them without concern for the refcount, which by
-			 * definition always must be one, so they don't check.
-			 */
-			continue;
+	ospc = WS_Reserve(wrk->aws, 0);
+	assert(ospc >= sizeof *ocp);
+	do {
+		more = 0;
+		spc = ospc;
+		nobj = 0;
+		ocp = (void*)wrk->aws->f;
+		Lck_Lock(&oh->mtx);
+		assert(oh->refcnt > 0);
+		now = VTIM_real();
+		VTAILQ_FOREACH(oc, &oh->objcs, list) {
+			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+			assert(oc->objhead == oh);
+			if (oc->flags & OC_F_BUSY) {
+				/*
+				 * We cannot purge busy objects here, because
+				 * their owners have special rights to them,
+				 * and may nuke them without concern for the
+				 * refcount, which by definition always must
+				 * be one, so they don't check.
+				 */
+				continue;
+			}
+			if (oc->exp_flags & OC_EF_DYING)
+				continue;
+			if (spc < sizeof *ocp) {
+				/* Iterate if aws is not big enough */
+				more = 1;
+				break;
+			}
+			oc->refcnt++;
+			spc -= sizeof *ocp;
+			ocp[nobj++] = oc;
 		}
-		if (oc->exp_flags & OC_EF_DYING)
-			continue;
-		xxxassert(spc >= sizeof *ocp);
-		oc->refcnt++;
-		spc -= sizeof *ocp;
-		ocp[nobj++] = oc;
-	}
-	Lck_Unlock(&oh->mtx);
+		Lck_Unlock(&oh->mtx);
 
-	for (n = 0; n < nobj; n++) {
-		oc = ocp[n];
-		CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-		o = oc_getobj(&wrk->stats, oc);
-		if (o == NULL)
-			continue;
-		CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
-		EXP_Rearm(o, now, ttl, grace, keep);
-		(void)HSH_DerefObj(&wrk->stats, &o);
-	}
+		for (n = 0; n < nobj; n++) {
+			oc = ocp[n];
+			CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
+			o = oc_getobj(&wrk->stats, oc);
+			if (o == NULL)
+				continue;
+			CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
+			EXP_Rearm(o, now, ttl, grace, keep);
+			(void)HSH_DerefObj(&wrk->stats, &o);
+		}
+	} while (more);
 	WS_Release(wrk->aws, 0);
 	Pool_PurgeStat(nobj);
 }
