@@ -87,6 +87,7 @@ DOT deliver:deliver:s -> DONE [style=bold,color=blue]
 static enum req_fsm_nxt
 cnt_deliver(struct worker *wrk, struct req *req)
 {
+	struct busyobj *bo;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -136,11 +137,22 @@ cnt_deliver(struct worker *wrk, struct req *req)
 	if (req->restarts >= cache_param->max_restarts)
 		wrk->handling = VCL_RET_DELIVER;
 
-	if (wrk->handling == VCL_RET_RESTART) {
+	if (wrk->handling != VCL_RET_DELIVER) {
 		(void)HSH_DerefObj(&wrk->stats, &req->obj);
 		AZ(req->obj);
 		http_Teardown(req->resp);
-		req->req_step = R_STP_RESTART;
+
+		switch (wrk->handling) {
+		case VCL_RET_RESTART:
+			req->req_step = R_STP_RESTART;
+			break;
+		case VCL_RET_SYNTH:
+			req->req_step = R_STP_SYNTH;
+			break;
+		default:
+			INCOMPL();
+		}
+
 		return (REQ_FSM_MORE);
 	}
 
@@ -154,7 +166,12 @@ cnt_deliver(struct worker *wrk, struct req *req)
 		req->wantbody = 0;
 	}
 
-	V1D_Deliver(req);
+	/* Grab a ref to the bo if there is one, and hand it down */
+	bo = HSH_RefBusy(req->obj->objcore);
+	V1D_Deliver(req, bo);
+	if (bo != NULL)
+		VBO_DerefBusyObj(req->wrk, &bo);
+
 	VSLb_ts_req(req, "Resp", W_TIM_real(wrk));
 
 	if (http_HdrIs(req->resp, H_Connection, "close"))
@@ -203,7 +220,6 @@ cnt_synth(struct worker *wrk, struct req *req)
 
 	wrk->stats.s_synth++;
 
-
 	now = W_TIM_real(wrk);
 	VSLb_ts_req(req, "Process", now);
 
@@ -241,6 +257,9 @@ cnt_synth(struct worker *wrk, struct req *req)
 	if (http_HdrIs(req->resp, H_Connection, "close"))
 		req->doclose = SC_RESP_CLOSE;
 
+	/* Discard any lingering request body before delivery */
+	(void)HTTP1_DiscardReqBody(req);
+
 	V1D_Deliver_Synth(req);
 
 	VSLb_ts_req(req, "Resp", W_TIM_real(wrk));
@@ -453,8 +472,7 @@ cnt_lookup(struct worker *wrk, struct req *req)
 
 	if (boc != NULL) {
 		(void)HSH_DerefObjCore(&wrk->stats, &boc);
-		free(req->vary_b);
-		req->vary_b = NULL;
+		VRY_Clear(req);
 	}
 
 	return (REQ_FSM_MORE);
@@ -508,7 +526,7 @@ cnt_miss(struct worker *wrk, struct req *req)
 	default:
 		WRONG("Illegal return from vcl_miss{}");
 	}
-	free(req->vary_b);
+	VRY_Clear(req);
 	if (o != NULL)
 		(void)HSH_DerefObj(&wrk->stats, &o);
 	AZ(HSH_DerefObjCore(&wrk->stats, &req->objcore));
@@ -700,6 +718,21 @@ cnt_recv(struct worker *wrk, struct req *req)
 
 	http_VSL_log(req->http);
 
+	if (req->restarts == 0) {
+		/*
+		 * This really should be done earlier, but we want to capture
+		 * it in the VSL log.
+		 */
+		if (http_GetHdr(req->http, H_X_Forwarded_For, &xff)) {
+			http_Unset(req->http, H_X_Forwarded_For);
+			http_PrintfHeader(req->http, "X-Forwarded-For: %s, %s",
+			    xff, req->sp->client_addr_str);
+		} else {
+			http_PrintfHeader(req->http, "X-Forwarded-For: %s",
+			    req->sp->client_addr_str);
+		}
+	}
+
 	if (req->err_code) {
 		req->req_step = R_STP_SYNTH;
 		return (REQ_FSM_MORE);
@@ -715,25 +748,15 @@ cnt_recv(struct worker *wrk, struct req *req)
 	req->hash_always_miss = 0;
 	req->hash_ignore_busy = 0;
 	req->client_identity = NULL;
-	if (req->restarts == 0) {
-		if (http_GetHdr(req->http, H_X_Forwarded_For, &xff)) {
-			http_Unset(req->http, H_X_Forwarded_For);
-			http_PrintfHeader(req->http, "X-Forwarded-For: %s, %s", xff,
-					  req->sp->client_addr_str);
-		} else {
-			http_PrintfHeader(req->http, "X-Forwarded-For: %s",
-					  req->sp->client_addr_str);
-		}
-	}
 
 	http_CollectHdr(req->http, H_Cache_Control);
 
 	VCL_recv_method(req->vcl, wrk, req, NULL, req->http->ws);
 
 	/* Attempts to cache req.body may fail */
-	if (req->req_body_status == REQ_BODY_FAIL) {
+	if (req->req_body_status == REQ_BODY_FAIL)
 		return (REQ_FSM_DONE);
-	}
+
 	recv_handling = wrk->handling;
 
 	/* We wash the A-E header here for the sake of VRY */
@@ -937,7 +960,7 @@ CNT_AcctLogCharge(struct dstat *ds, struct req *req)
 
 	a = &req->acct;
 
-	if (!(req->res_mode & RES_PIPE)) {
+	if (req->vsl->wid && !(req->res_mode & RES_PIPE)) {
 		VSLb(req->vsl, SLT_ReqAcct, "%ju %ju %ju %ju %ju %ju",
 		    (uintmax_t)a->req_hdrbytes,
 		    (uintmax_t)a->req_bodybytes,
