@@ -30,15 +30,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "vtc.h"
 
-#include "vqueue.h"
-#include "miniobj.h"
-#include "libvarnish.h"
-
 struct sema {
 	unsigned		magic;
 #define SEMA_MAGIC		0x29b64317
@@ -67,7 +63,7 @@ sema_new(char *name, struct vtclog *vl)
 	AN(r);
 	r->name = name;
 	if (*name != 'r')
-		vtc_log(vl, 0, "Sema name must start with 'r' (%s)", *name);
+		vtc_log(vl, 0, "Sema name must start with 'r' (%s)", name);
 
 	AZ(pthread_mutex_init(&r->mtx, NULL));
 	AZ(pthread_cond_init(&r->cond, NULL));
@@ -133,7 +129,7 @@ cmd_sema(CMD_ARGS)
 		return;
 	}
 
-	assert(!strcmp(av[0], "sema"));
+	AZ(strcmp(av[0], "sema"));
 	av++;
 
 	AZ(pthread_mutex_lock(&sema_mtx));
