@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2010 Varnish Software AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -28,17 +28,21 @@
  *
  * Implementation of a binary heap API
  *
- * We use a malloc(3)/realloc(3) array to store the pointers using the
- * classical FORTRAN strategy.
+ * See also:
+ *	http://portal.acm.org/citation.cfm?doid=1785414.1785434
+ *	(or: http://queue.acm.org/detail.cfm?id=1814327)
  */
 
 #include "config.h"
 
-#include <unistd.h>
+#include <errno.h>
+#include <limits.h>
+#include <stdint.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include "binary_heap.h"
-#include "libvarnish.h"
+#include "vas.h"
 
 /* Parameters --------------------------------------------------------*/
 
@@ -96,6 +100,7 @@ parent(const struct binheap *bh, unsigned u)
 	unsigned po;
 	unsigned v;
 
+	assert(u != UINT_MAX);
 	po = u & bh->page_mask;
 
 	if (u < bh->page_size || po > 3) {
@@ -113,6 +118,7 @@ parent(const struct binheap *bh, unsigned u)
 static void
 child(const struct binheap *bh, unsigned u, unsigned *a, unsigned *b)
 {
+	uintmax_t uu;
 
 	if (u > bh->page_mask && (u & (bh->page_mask - 1)) == 0) {
 		/* First two elements are magical except on the first page */
@@ -122,16 +128,32 @@ child(const struct binheap *bh, unsigned u, unsigned *a, unsigned *b)
 		*a = (u & ~bh->page_mask) >> 1;
 		*a |= u & (bh->page_mask >> 1);
 		*a += 1;
-		*a <<= bh->page_shift;
-		*b = *a + 1;
+		uu = (uintmax_t)*a << bh->page_shift;
+		*a = uu;
+		if (*a == uu) {
+			*b = *a + 1;
+		} else {
+			/*
+			 * An unsigned is not big enough: clamp instead
+			 * of truncating.  We do not support adding
+			 * more than UINT_MAX elements anyway, so this
+			 * is without consequence.
+			 */
+			*a = UINT_MAX;
+			*b = UINT_MAX;
+		}
 	} else {
 		/* The rest is as usual, only inside the page */
 		*a = u + (u & bh->page_mask);
 		*b = *a + 1;
 	}
 #ifdef PARANOIA
-	assert(parent(bh, *a) == u);
-	assert(parent(bh, *b) == u);
+	assert(*a > 0);
+	assert(*b > 0);
+	if (*a != UINT_MAX) {
+		assert(parent(bh, *a) == u);
+		assert(parent(bh, *b) == u);
+	}
 #endif
 }
 
@@ -193,7 +215,7 @@ binheap_new(void *priv, binheap_cmp_t *cmp_f, binheap_update_t *update_f)
 
 	bh->page_size = (unsigned)getpagesize() / sizeof (void *);
 	bh->page_mask = bh->page_size - 1;
-	assert(!(bh->page_size & bh->page_mask));	/* power of two */
+	AZ(bh->page_size & bh->page_mask);	/* power of two */
 	for (u = 1; (1U << u) != bh->page_size; u++)
 		;
 	bh->page_shift = u;
@@ -214,12 +236,12 @@ binheap_new(void *priv, binheap_cmp_t *cmp_f, binheap_update_t *update_f)
 static void
 binheap_update(const struct binheap *bh, unsigned u)
 {
+	assert(bh != NULL);
 	assert(bh->magic == BINHEAP_MAGIC);
 	assert(u < bh->next);
 	assert(A(bh, u) != NULL);
-	if (bh->update == NULL)
-		return;
-	bh->update(bh->priv, A(bh, u), u);
+	if (bh->update != NULL)
+		bh->update(bh->priv, A(bh, u), u);
 }
 
 static void
@@ -227,9 +249,12 @@ binhead_swap(const struct binheap *bh, unsigned u, unsigned v)
 {
 	void *p;
 
+	assert(bh != NULL);
 	assert(bh->magic == BINHEAP_MAGIC);
 	assert(u < bh->next);
+	assert(A(bh, u) != NULL);
 	assert(v < bh->next);
+	assert(A(bh, v) != NULL);
 	p = A(bh, u);
 	A(bh, u) = A(bh, v);
 	A(bh, v) = p;
@@ -242,9 +267,17 @@ binheap_trickleup(const struct binheap *bh, unsigned u)
 {
 	unsigned v;
 
-	assert(bh->magic == BINHEAP_MAGIC);
+	assert(bh != NULL); assert(bh->magic == BINHEAP_MAGIC);
+	assert(u < bh->next);
+	assert(A(bh, u) != NULL);
+
 	while (u > ROOT_IDX) {
+		assert(u < bh->next);
+		assert(A(bh, u) != NULL);
 		v = parent(bh, u);
+		assert(v < u);
+		assert(v < bh->next);
+		assert(A(bh, v) != NULL);
 		if (!bh->cmp(bh->priv, A(bh, u), A(bh, v)))
 			break;
 		binhead_swap(bh, u, v);
@@ -253,20 +286,37 @@ binheap_trickleup(const struct binheap *bh, unsigned u)
 	return (u);
 }
 
-static void
+static unsigned
 binheap_trickledown(const struct binheap *bh, unsigned u)
 {
 	unsigned v1, v2;
 
+	assert(bh != NULL);
 	assert(bh->magic == BINHEAP_MAGIC);
+	assert(u < bh->next);
+	assert(A(bh, u) != NULL);
+
 	while (1) {
+		assert(u < bh->next);
+		assert(A(bh, u) != NULL);
 		child(bh, u, &v1, &v2);
+		assert(v1 > 0);
+		assert(v2 > 0);
+		assert(v1 <= v2);
+
 		if (v1 >= bh->next)
-			return;
-		if (v2 < bh->next && bh->cmp(bh->priv, A(bh, v2), A(bh, v1)))
-			v1 = v2;
+			return (u);
+
+		assert(A(bh, v1) != NULL);
+		if (v1 != v2 && v2 < bh->next) {
+			assert(A(bh, v2) != NULL);
+			if (bh->cmp(bh->priv, A(bh, v2), A(bh, v1)))
+				v1 = v2;
+		}
+		assert(v1 < bh->next);
+		assert(A(bh, v1) != NULL);
 		if (bh->cmp(bh->priv, A(bh, u), A(bh, v1)))
-			return;
+			return (u);
 		binhead_swap(bh, u, v1);
 		u = v1;
 	}
@@ -282,10 +332,13 @@ binheap_insert(struct binheap *bh, void *p)
 	assert(bh->length >= bh->next);
 	if (bh->length == bh->next)
 		binheap_addrow(bh);
+	assert(bh->length > bh->next);
 	u = bh->next++;
 	A(bh, u) = p;
 	binheap_update(bh, u);
 	(void)binheap_trickleup(bh, u);
+	assert(u < bh->next);
+	assert(A(bh, u) != NULL);
 }
 
 
@@ -297,7 +350,7 @@ chk(const struct binheap *bh)
 
 	for (u = 2; u < bh->next; u++) {
 		v = parent(bh, u);
-		assert(!bh->cmp(bh->priv, A(bh, u), A(bh, v)));
+		AZ(bh->cmp(bh->priv, A(bh, u), A(bh, v)));
 	}
 }
 #endif
@@ -331,11 +384,11 @@ binheap_root(const struct binheap *bh)
  * N{height}-1 upward trickles.
  *
  * When we fill the hole with the tail object, the worst case is
- * that it trickles all the way down to become the tail object
- * again.
- * In other words worst case is N{height} downward trickles.
- * But there is a pretty decent chance that it does not make
- * it all the way down.
+ * that it trickles all the way up to of this half-tree, or down
+ * to become the tail object again.
+ *
+ * In other words worst case is N{height} up or downward trickles.
+ * But there is a decent chance that it does not make it all the way.
  */
 
 void
@@ -357,7 +410,13 @@ binheap_delete(struct binheap *bh, unsigned idx)
 	A(bh, bh->next) = NULL;
 	binheap_update(bh, idx);
 	idx = binheap_trickleup(bh, idx);
-	binheap_trickledown(bh, idx);
+	assert(idx < bh->next);
+	assert(idx > 0);
+	assert(A(bh, idx) != NULL);
+	idx = binheap_trickledown(bh, idx);
+	assert(idx < bh->next);
+	assert(idx > 0);
+	assert(A(bh, idx) != NULL);
 
 	/*
 	 * We keep a hysteresis of one full row before we start to
@@ -386,31 +445,60 @@ binheap_reorder(const struct binheap *bh, unsigned idx)
 	assert(idx > 0);
 	assert(A(bh, idx) != NULL);
 	idx = binheap_trickleup(bh, idx);
-	binheap_trickledown(bh, idx);
+	assert(idx < bh->next);
+	assert(idx > 0);
+	assert(A(bh, idx) != NULL);
+	idx = binheap_trickledown(bh, idx);
+	assert(idx < bh->next);
+	assert(idx > 0);
+	assert(A(bh, idx) != NULL);
 }
 
 #ifdef TEST_DRIVER
-/* Test driver -------------------------------------------------------*/
+
 #include <stdio.h>
 
+#include "miniobj.h"
+
+/* Test driver -------------------------------------------------------*/
+
+static void
+vasfail(const char *func, const char *file, int line,
+    const char *cond, int err, int xxx)
+{
+	fprintf(stderr, "PANIC: %s %s %d %s %d %d\n",
+		func, file, line, cond, err, xxx);
+	abort();
+}
+
+vas_f *VAS_Fail = vasfail;
+
 struct foo {
+	unsigned	magic;
+#define FOO_MAGIC	0x23239823
 	unsigned	idx;
 	unsigned	key;
+	unsigned	n;
 };
 
+#if 1
 #define M 31011091	/* Number of operations */
-#define N 10313102		/* Number of items */
+#define N 17313102	/* Number of items */
+#else
+#define M 3401		/* Number of operations */
+#define N 1131		/* Number of items */
+#endif
 #define R -1		/* Random modulus */
 
-struct foo ff[N];
+struct foo *ff[N];
 
 static int
 cmp(void *priv, void *a, void *b)
 {
 	struct foo *fa, *fb;
 
-	fa = a;
-	fb = b;
+	CAST_OBJ_NOTNULL(fa, a, FOO_MAGIC);
+	CAST_OBJ_NOTNULL(fb, b, FOO_MAGIC);
 	return (fa->key < fb->key);
 }
 
@@ -419,12 +507,12 @@ update(void *priv, void *a, unsigned u)
 {
 	struct foo *fa;
 
-	fa = a;
+	CAST_OBJ_NOTNULL(fa, a, FOO_MAGIC);
 	fa->idx = u;
 }
 
 void
-chk(struct binheap *bh)
+chk2(struct binheap *bh)
 {
 	unsigned u, v;
 	struct foo *fa, *fb;
@@ -441,7 +529,7 @@ int
 main(int argc, char **argv)
 {
 	struct binheap *bh;
-	unsigned u, v, lr;
+	unsigned u, v, lr, n;
 	struct foo *fp;
 
 	if (0) {
@@ -451,59 +539,93 @@ main(int argc, char **argv)
 		srandom(u);
 	}
 	bh = binheap_new(NULL, cmp, update);
-
-	/* First insert our N elements */
-	for (u = 0; u < N; u++) {
-		lr = random() % R;
-		ff[u].key = lr;
-		binheap_insert(bh, &ff[u]);
-
-		fp = binheap_root(bh);
-		assert(fp->idx == 1);
-		assert(fp->key <= lr);
+	for (n = 2; n; n += n) {
+		child(bh, n - 1, &u, &v);
+		child(bh, n, &u, &v);
+		child(bh, n + 1, &u, &v);
 	}
-	fprintf(stderr, "%d inserts OK\n", N);
-	/* For M cycles, pick the root, insert new */
-	for (u = 0; u < M; u++) {
-		fp = binheap_root(bh);
-		assert(fp->idx == 1);
 
-		/* It cannot possibly be larger than the last value we added */
-		assert(fp->key <= lr);
-		binheap_delete(bh, fp->idx);
+	while (1) {
+		/* First insert our N elements */
+		for (u = 0; u < N; u++) {
+			lr = random() % R;
+			ALLOC_OBJ(ff[u], FOO_MAGIC);
+			assert(ff[u] != NULL);
+			ff[u]->key = lr;
+			ff[u]->n = u;
+			binheap_insert(bh, ff[u]);
 
-		lr = random() % R;
-		fp->key = lr;
-		binheap_insert(bh, fp);
-	}
-	fprintf(stderr, "%d replacements OK\n", M);
-	/* The remove everything */
-	lr = 0;
-	for (u = 0; u < N; u++) {
-		fp = binheap_root(bh);
-		assert(fp->idx == 1);
-		assert(fp->key >= lr);
-		lr = fp->key;
-		binheap_delete(bh, fp->idx);
-	}
-	fprintf(stderr, "%d removes OK\n", N);
-
-	for (u = 0; u < M; u++) {
-		v = random() % N;
-		if (ff[v].idx > 0) {
-			assert(ff[v].idx != 0);
-			binheap_delete(bh, ff[v].idx);
-			assert(ff[v].idx == 0);
-		} else {
-			assert(ff[v].idx == 0);
-			ff[v].key = random() % R;
-			binheap_insert(bh, &ff[v]);
-			assert(ff[v].idx != 0);
+			fp = binheap_root(bh);
+			assert(fp->idx == 1);
+			assert(fp->key <= lr);
 		}
-		if (0)
-			chk(bh);
+		fprintf(stderr, "%d inserts OK\n", N);
+		/* For M cycles, pick the root, insert new */
+		for (u = 0; u < M; u++) {
+			fp = binheap_root(bh);
+			CHECK_OBJ_NOTNULL(fp, FOO_MAGIC);
+			assert(fp->idx == 1);
+
+			/*
+			 * It cannot possibly be larger than the last
+			 * value we added
+			 */
+			assert(fp->key <= lr);
+			binheap_delete(bh, fp->idx);
+
+			n = fp->n;
+			ALLOC_OBJ(ff[n], FOO_MAGIC);
+			assert(ff[n] != NULL);
+			FREE_OBJ(fp);
+			fp = ff[n];
+			fp->n = n;
+
+			lr = random() % R;
+			fp->key = lr;
+			binheap_insert(bh, fp);
+		}
+		fprintf(stderr, "%d replacements OK\n", M);
+		/* The remove everything */
+		lr = 0;
+		for (u = 0; u < N; u++) {
+			fp = binheap_root(bh);
+			CHECK_OBJ_NOTNULL(fp, FOO_MAGIC);
+			assert(fp->idx == 1);
+			assert(fp->key >= lr);
+			lr = fp->key;
+			binheap_delete(bh, fp->idx);
+			ff[fp->n] = NULL;
+			FREE_OBJ(fp);
+		}
+		fprintf(stderr, "%d removes OK\n", N);
+
+		for (u = 0; u < M; u++) {
+			v = random() % N;
+			if (ff[v] != NULL) {
+				CHECK_OBJ_NOTNULL(ff[v], FOO_MAGIC);
+				AN(ff[v]->idx);
+				if (ff[v]->key & 1) {
+					binheap_delete(bh, ff[v]->idx);
+					assert(ff[v]->idx == BINHEAP_NOIDX);
+					FREE_OBJ(ff[v]);
+					ff[v] = NULL;
+				} else {
+					ff[v]->key = random() % R;
+					binheap_reorder(bh, ff[v]->idx);
+				}
+			} else {
+				ALLOC_OBJ(ff[v], FOO_MAGIC);
+				assert(ff[v] != NULL);
+				ff[v]->key = random() % R;
+				binheap_insert(bh, ff[v]);
+				CHECK_OBJ_NOTNULL(ff[v], FOO_MAGIC);
+				AN(ff[v]->idx);
+			}
+			if (0)
+				chk2(bh);
+		}
+		fprintf(stderr, "%d updates OK\n", M);
 	}
-	fprintf(stderr, "%d updates OK\n", M);
 	return (0);
 }
 #endif
