@@ -36,19 +36,19 @@
 #include <poll.h>
 #include <signal.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 
-#include "vsb.h"
-
-#include "libvarnish.h"
-
+#include "miniobj.h"
+#include "vas.h"
 #include "vcli.h"
-#include "cli_priv.h"
-#include "cli_common.h"
+#include "vcli_common.h"
+#include "vcli_priv.h"
+#include "vsb.h"
 
 /*lint -e{818} cli could be const */
 void
@@ -57,18 +57,35 @@ VCLI_Out(struct cli *cli, const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	if (cli != NULL)
-		(void)VSB_vprintf(cli->sb, fmt, ap);
-	else
+	if (cli != NULL) {
+		CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
+		if (VSB_len(cli->sb) < *cli->limit)
+			(void)VSB_vprintf(cli->sb, fmt, ap);
+		else if (cli->result == CLIS_OK)
+			cli->result = CLIS_TRUNCATED;
+	} else {
 		(void)vfprintf(stdout, fmt, ap);
+	}
 	va_end(ap);
 }
 
+/*lint -e{818} cli could be const */
+int
+VCLI_Overflow(struct cli *cli)
+{
+	CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
+	if (cli->result == CLIS_TRUNCATED ||
+	    VSB_len(cli->sb) >= *cli->limit)
+		return (1);
+	return (0);
+}
+
 /*lint -e{818} cli could be const */
 void
 VCLI_Quote(struct cli *cli, const char *s)
 {
 
+	CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
 	VSB_quote(cli->sb, s, -1, 0);
 }
 
@@ -76,10 +93,13 @@ void
 VCLI_SetResult(struct cli *cli, unsigned res)
 {
 
-	if (cli != NULL)
-		cli->result = res;	/*lint !e64 type mismatch */
-	else
+	if (cli != NULL) {
+		CHECK_OBJ_NOTNULL(cli, CLI_MAGIC);
+		if (cli->result != CLIS_TRUNCATED || res != CLIS_OK)
+			cli->result = res;	/*lint !e64 type mismatch */
+	} else {
 		printf("CLI result = %u\n", res);
+	}
 }
 
 int
@@ -88,6 +108,7 @@ VCLI_WriteResult(int fd, unsigned status, const char *result)
 	int i, l;
 	struct iovec iov[3];
 	char nl[2] = "\n";
+	size_t len;
 	char res[CLI_LINE0_LEN + 2];	/*
 					 * NUL + one more so we can catch
 					 * any misformats by snprintf
@@ -96,15 +117,18 @@ VCLI_WriteResult(int fd, unsigned status, const char *result)
 	assert(status >= 100);
 	assert(status <= 999);		/*lint !e650 const out of range */
 
+	len = strlen(result);
+
 	i = snprintf(res, sizeof res,
-	    "%-3d %-8jd\n", status, (intmax_t)strlen(result));
+	    "%-3d %-8zd\n", status, len);
 	assert(i == CLI_LINE0_LEN);
+	assert(strtoul(res + 3, NULL, 10) == len);
 
 	iov[0].iov_base = res;
 	iov[0].iov_len = CLI_LINE0_LEN;
 
 	iov[1].iov_base = (void*)(uintptr_t)result;	/* TRUST ME */
-	iov[1].iov_len = strlen(result);
+	iov[1].iov_len = len;
 
 	iov[2].iov_base = nl;
 	iov[2].iov_len = 1;
@@ -122,7 +146,7 @@ read_tmo(int fd, char *ptr, unsigned len, double tmo)
 	struct pollfd pfd;
 
 	if (tmo > 0)
-		to = tmo * 1e3;
+		to = (int)(tmo * 1e3);
 	else
 		to = -1;
 	pfd.fd = fd;
@@ -151,44 +175,56 @@ VCLI_ReadResult(int fd, unsigned *status, char **ptr, double tmo)
 	char res[CLI_LINE0_LEN];	/* For NUL */
 	int i, j;
 	unsigned u, v, s;
-	char *p;
+	char *p = NULL;
+	const char *err = "CLI communication error (hdr)";
 
 	if (status == NULL)
 		status = &s;
 	if (ptr != NULL)
 		*ptr = NULL;
-	i = read_tmo(fd, res, CLI_LINE0_LEN, tmo);
-	if (i != CLI_LINE0_LEN) {
-		*status = CLIS_COMMS;
-		if (ptr != NULL)
-			*ptr = strdup("CLI communication error (hdr)");
-		if (i != 0)
-			return (i);
-		return (*status);
-	}
-	assert(i == CLI_LINE0_LEN);
-	assert(res[3] == ' ');
-	assert(res[CLI_LINE0_LEN - 1] == '\n');
-	res[CLI_LINE0_LEN - 1] = '\0';
-	j = sscanf(res, "%u %u\n", &u, &v);
-	assert(j == 2);
-	*status = u;
-	p = malloc(v + 1L);
-	assert(p != NULL);
-	i = read_tmo(fd, p, v + 1, tmo);
-	if (i < 0) {
-		*status = CLIS_COMMS;
+	do {
+		i = read_tmo(fd, res, CLI_LINE0_LEN, tmo);
+		if (i != CLI_LINE0_LEN)
+			break;
+
+		if (res[3] != ' ')
+			break;
+
+		if (res[CLI_LINE0_LEN - 1] != '\n')
+			break;
+
+		res[CLI_LINE0_LEN - 1] = '\0';
+		j = sscanf(res, "%u %u\n", &u, &v);
+		if (j != 2)
+			break;
+
+		err = "CLI communication error (body)";
+
+		*status = u;
+		p = malloc(v + 1L);
+		if (p == NULL)
+			break;
+
+		i = read_tmo(fd, p, v + 1, tmo);
+		if (i < 0)
+			break;
+		if (i != v + 1)
+			break;
+		if (p[v] != '\n')
+			break;
+
+		p[v] = '\0';
+		if (ptr == NULL)
+			free(p);
+		else
+			*ptr = p;
+		return (0);
+	} while(0);
+
+	if (p != NULL)
 		free(p);
-		if (ptr != NULL)
-			*ptr = strdup("CLI communication error (body)");
-		return (i);
-	}
-	assert(i == v + 1);
-	assert(p[v] == '\n');
-	p[v] = '\0';
-	if (ptr == NULL)
-		free(p);
-	else
-		*ptr = p;
-	return (0);
+	*status = CLIS_COMMS;
+	if (ptr != NULL)
+		*ptr = strdup(err);
+	return (*status);
 }
