@@ -43,8 +43,9 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "libvarnish.h"
+#include "vas.h"
 #include "vss.h"
+#include "vtcp.h"
 
 /* lightweight addrinfo */
 struct vss_addr {
@@ -70,7 +71,7 @@ struct vss_addr {
  * See also RFC5952
  */
 
-int
+const char *
 VSS_parse(const char *str, char **addr, char **port)
 {
 	const char *p;
@@ -82,7 +83,7 @@ VSS_parse(const char *str, char **addr, char **port)
 		if ((p = strchr(str, ']')) == NULL ||
 		    p == str + 1 ||
 		    (p[1] != '\0' && p[1] != ':'))
-			return (-1);
+			return ("IPv6 address [] wrong.");
 		*addr = strdup(str + 1);
 		XXXAN(*addr);
 		(*addr)[p - (str + 1)] = '\0';
@@ -108,7 +109,7 @@ VSS_parse(const char *str, char **addr, char **port)
 			XXXAN(*port);
 		}
 	}
-	return (0);
+	return (NULL);
 }
 
 /*
@@ -134,6 +135,7 @@ VSS_resolve(const char *addr, const char *port, struct vss_addr ***vap)
 	struct addrinfo hints, *res0, *res;
 	struct vss_addr **va;
 	int i, ret;
+	long int ptst;
 	char *adp, *hop;
 
 	*vap = NULL;
@@ -141,14 +143,20 @@ VSS_resolve(const char *addr, const char *port, struct vss_addr ***vap)
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_PASSIVE;
 
-	ret = VSS_parse(addr, &hop, &adp);
-	if (ret)
+	if (VSS_parse(addr, &hop, &adp) != NULL)
 		return (0);
 
 	if (adp == NULL)
 		ret = getaddrinfo(addr, port, &hints, &res0);
-	else
+	else {
+		ptst = strtol(adp, NULL, 10);
+		if (ptst < 0 || ptst > 65535) {
+			free(hop);
+			free(adp);
+			return(0);
+		}
 		ret = getaddrinfo(hop, adp, &hints, &res0);
+	}
 
 	free(hop);
 	free(adp);
